/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ws3dproxy.model;

import ws3dproxy.util.Constants;
import java.awt.Color;
import java.awt.geom.Rectangle2D;
import ws3dproxy.DrawingThingManager;

/**
 *
 * @author eccastro
 */
//public abstract class Thing {
public class Thing {

    //attibutes also stored in "RememberedThing" (aka ObjectData)
    protected double x1, y1;
    protected double x2, y2;
    protected String name;
    public int category = -1;

    //currently not set when retrieved from memory
    public double pitch = -1; 
    private Material3D material;
    public int isOccluded = 0; //0-false; 1-true

    private boolean fromMemory = false;

    public boolean hidden = false;
    public int ifIsADeliverySot = 0;
    protected WorldPoint center;
    public WorldPoint[] vertex = new WorldPoint[4]; //formed by waypoints
    //Currently, it happens to be the same as vertex (previous array). It may be different if "GAP" is different from "SEC".
    public WorldPoint[] secAreaVertex = new WorldPoint[4]; //this is the security area around any thing
    private Rectangle2D.Double recSecArea;
    public Rectangle2D.Double recGapArea;
    public Rectangle2D.Double minVSArea;

    private DrawingThingManager drawer;

    /**
     * Used only to "recreate" a "remembered Thing".
     * @param name
     * @param category
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param colorName
     */
    public Thing(String name, int category, double x1, double y1, double x2, double y2, String colorName) {
        this.x1 = x1;
        this.x2 = x2;
        this.y2 = y2;
        this.y1 = y1;
        this.category = category;
        this.name = name;
        this.center = new WorldPoint((x1 + x2) / 2, (y1 + y2) / 2);
        this.material = new Material3D(-1, -1, -1, translateIntoColor(colorName));// -1 means undefined

        this.fromMemory = true;
        this.drawer = new DrawingThingManager(this);
        
    }
    public Thing(String name, int category, double x1, double y1, double x2, double y2, String colorName, int isOccluded, double pitch, double hardness, double energy, double taste) {
        this(name, category, x1, y1, x2, y2, colorName);
    
        this.isOccluded = isOccluded;
        this.pitch = pitch;
        this.material = new Material3D(hardness, energy, taste, translateIntoColor(colorName));

        this.fromMemory = false;
    }

    public double getX1() {
        return x1;
    }

    public double getY1() {
        return y1;
    }

    public double getX2() {
        return x2;
    }

    public double getY2() {
        return y2;
    }

    public double getHeight() {
        return y2 - y1;
    }

    public double getWidth() {
        return x2 - x1;
    }

    public double getPitch() {
        return pitch;
    }

    public void setX1(double x1) {
        this.x1 = x1;
    }

    public void setY1(double y1) {
        this.y1 = y1;
    }

    public void setX2(double x2) {
        this.x2 = x2;
    }

    public void setY2(double y2) {
        this.y2 = y2;
    }

    public boolean getIfFromMemory() {
        return this.fromMemory;
    }

    public DrawingThingManager getDrawer() {
        return this.drawer;
    }

    public Material3D getMaterial() {
        return material;
    }

    //public abstract void setID(Long id);
    public void setName(String id) {
        this.name = id;
    }

    public String getName() {
        return this.name;
    }

    //currently not in use!
    public boolean containsPoint(WorldPoint p) {
        double xx = p.getX();
        double yy = p.getY();
        return this.containsPoint(xx, yy);
        //return this.withinBoundingArea(xx, yy);
    }

    public boolean containsPoint(double x, double y) {
        return (x1 <= x) && (x2 >= x) && (y1 <= y) && (y2 >= y);

    }

    /**
     * Checks if point is inside the security area around (and including) the thing.
     *
     * @param x
     * @param y
     * @return
     */
    public boolean innerWayPointAreaContainsPoint(double x, double y) {

        WorldPoint[] points = this.initAndGetWayPoints();
        this.getSecArea();

        for (int i = 0; i < points.length; i++) {
            if (points[i].getX() == x && points[i].getY() == y) {
                return false;
            }
        }

        //System.out.println("------- contains= " + this.recGapArea.contains(x, y));
        //return this.recGapArea.contains(x, y);

        //System.out.println("------- contains= " + this.recSecArea.contains(x, y));
        return this.getSecArea().contains(x, y);
    }

    /**
     * Checks if point is inside the gap area around (and including) the thing.
     * Its vertex (or waypoints) ARE considered.
     * @param x
     * @param y
     * @return
     */
    public boolean isInsideGapArea(double x, double y) {

        // initAndGetWayPoints();
        boolean checkBoundary = false;
        WorldPoint[] points = this.initAndGetWayPoints();
        for (int i = 0; i < points.length; i++) {
            if (points[i].getX() == x && points[i].getY() == y) {
                checkBoundary = true;
            }
        }
        if (checkBoundary) {
            return true;
        } else {
            return this.recGapArea.contains(x, y);
        }
//        this.initAndGetWayPoints();
//
        //System.out.println("---isMyWaypoint---- contains= " + this.recGapArea.contains(x, y)+ " x= "+x+" y= "+y);

        //return (points[0].getX() <= x) && (points[3].getX() >= x) && ( points[0].getY()<= y) && (points[3].getY() >= y);

    }

    /**
     * Currrently not in use. Use: innerWayPointAreaContainsPoint  instead
     * @param x
     * @param y
     * @return
     */
    public boolean withinBoundingArea(double x, double y) {
        double xx1, xx2, yy1, yy2;
        xx1 = this.getX1() - Constants.GAP;
        xx2 = this.getX2() + Constants.GAP;
        yy1 = this.getY1() - Constants.GAP;
        yy2 = this.getY2() + Constants.GAP;

        return (xx1 <= x) && (xx2 >= x) && (yy1 <= y) && (yy2 >= y);

    }

    public synchronized WorldPoint getCenterPosition() {
        return center;
    }

    public synchronized void setCenterPosition(WorldPoint c) {
        center = c;
    }

    public synchronized WorldPoint[] initAndGetWayPoints() {

        double gap = Constants.GAP;


        vertex[0] = new WorldPoint(this.getX1() - gap, this.getY1() - gap);
        vertex[1] = new WorldPoint(this.getX1() - gap, this.getY2() + gap); //upper left
        vertex[2] = new WorldPoint(this.getX2() + gap, this.getY1() - gap);
        vertex[3] = new WorldPoint(this.getX2() + gap, this.getY2() + gap);

        // vertex[1] is the upper left of the gap area
        double height = Math.abs(vertex[3].getX() - vertex[1].getX());
        double width = Math.abs(vertex[1].getY() - vertex[0].getY());

        this.recGapArea = new Rectangle2D.Double(vertex[0].getX(), vertex[0].getY(), height, width);

        return vertex;
    }

    public Rectangle2D.Double getSecArea() {


        WorldPoint[] insideSecArea = new WorldPoint[4];


        insideSecArea[0] = new WorldPoint(this.getX1() - Constants.SEC, this.getY1() - Constants.SEC);
        insideSecArea[1] = new WorldPoint(this.getX1() - Constants.SEC, this.getY2() + Constants.SEC);
        insideSecArea[2] = new WorldPoint(this.getX2() + Constants.SEC, this.getY1() - Constants.SEC );
        insideSecArea[3] = new WorldPoint(this.getX2() + Constants.SEC, this.getY2() + Constants.SEC);


        double height = Math.abs(insideSecArea[3].getX() - insideSecArea[1].getX());
        double width = Math.abs(insideSecArea[1].getY() - insideSecArea[0].getY());

        this.recSecArea = new Rectangle2D.Double(insideSecArea[0].getX(), insideSecArea[0].getY(), height, width);
        initMinimumVSArea();
        return this.recSecArea;
    }

    public boolean secAreaContain(double x, double y) {
        this.getSecArea();
        return this.getSecArea().contains(x, y);
    }
    
    public void initMinimumVSArea() {

        double factor = 5;

        WorldPoint[] minimumVSArea = new WorldPoint[4];


        minimumVSArea[0] = new WorldPoint(this.getX1() + factor, this.getY1() + factor);
        minimumVSArea[1] = new WorldPoint(this.getX1() + factor, this.getY2() - factor);
        minimumVSArea[2] = new WorldPoint(this.getX2() - factor, this.getY1() + factor);
        minimumVSArea[3] = new WorldPoint(this.getX2() - factor, this.getY2() - factor);


        double height = Math.abs(minimumVSArea[3].getX() - minimumVSArea[1].getX());
        double width = Math.abs(minimumVSArea[1].getY() - minimumVSArea[0].getY());

        this.minVSArea = new Rectangle2D.Double(minimumVSArea[0].getX(), minimumVSArea[0].getY(), height, width);

    }

    private Color translateIntoColor(String name) {
        return Constants.translateIntoColor(name);
    }

        public String toString(){
            return this.getName();
        }

}
