/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ws3dproxy.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;
import ws3dproxy.CommandExecException;
import ws3dproxy.CommandUtility;
import ws3dproxy.resourcesgenerator.ResourcesGenerator;
import ws3dproxy.util.Constants;

/**
 * The World or "environment" known by the Creature
 *
 * @author eccastro
 */
public class World {

    private static int environmentWidth;
    private static int environmentHeight;
    private static World instance = null;
    private static WorldPoint deliverySpot;
    static final Logger logger = Logger.getLogger(World.class.getName());
    //All Things except myself
    private HashMap<String, Thing> allThings = new HashMap();

    private World() {
    }

    public static World getInstance() {
        if (instance == null) {
            instance = new World();
        }

        return instance;
    }

    public synchronized int getEnvironmentWidth() {
        return environmentWidth;
    }

    public synchronized void setEnvironmentWidth(int environmentWidth) {
        this.environmentWidth = environmentWidth;
    }

    public synchronized int getEnvironmentHeight() {
        return environmentHeight;
    }

    public synchronized void setEnvironmentHeight(int environmentHeight) {
        this.environmentHeight = environmentHeight;
    }

    public String toString() {
        StringBuffer s = new StringBuffer();
        s.append("Simulation Parameters\n");
        s.append("Environment width: " + environmentWidth + " and height: "
                + environmentHeight + "\n");
        return s.toString();
    }

    public synchronized HashMap<String, Thing> getThingsMap() {

        return this.allThings;
    }

    public synchronized void updateThing(Thing th) {

        this.allThings.put(th.getName(), th);
    }

    public synchronized void removeThingFromMap(String name) {

        this.allThings.remove(name);
    }

    public synchronized List<Thing> getEveryThing() {

        return new ArrayList(this.allThings.values());
    }

    /**
     * Get the World's width and height and also the position (xDS, yDS) of the
     * DeliverySpot.
     *
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void getDimensionAndDeliverySpot() throws CommandExecException {
        StringTokenizer st = CommandUtility.sendGetSimulationParameters();
        double xds, yds;

        if (st.hasMoreTokens()) {
            World.environmentWidth = Integer.parseInt(st.nextToken());
        } else {
            java.util.logging.Logger.getLogger(World.class.getName()).log(Level.SEVERE, null, "Error: missing World width");
            return;
        }
        if (st.hasMoreTokens()) {
            World.environmentHeight = Integer.parseInt(st.nextToken());
        } else {
            java.util.logging.Logger.getLogger(World.class.getName()).log(Level.SEVERE, null, "Error: missing World height");
            return;
        }
        if (st.hasMoreTokens()) {
            xds = Double.parseDouble(st.nextToken());
        } else {
            java.util.logging.Logger.getLogger(World.class.getName()).log(Level.SEVERE, null, "Error: missing x of DeliverySpot");
            return;
        }
        if (st.hasMoreTokens()) {
            yds = Double.parseDouble(st.nextToken());
            World.deliverySpot = new WorldPoint(xds, yds);
        } else {
            java.util.logging.Logger.getLogger(World.class.getName()).log(Level.SEVERE, null, "Error: missing y of DeliverySpot");
            return;
        }

    }

    /**
     * Command to create a visual reference (e.g. arrow) at a specific point
     * within the server environment and expressed in Cartesian coordinate
     * system (x,y). Each point within the server environment is called a
     * "waypoint"
     *
     * @param x abscissa of the point
     * @param y ordinate of the point
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public synchronized void createWaypoint(double x, double y) throws CommandExecException {
        CommandUtility.sendNewWaypoint(x, y);

    }

    /**
     * Command to set the simulation game environment dimension: width, height
     * and the texture for the simulation floor.
     *
     * @param width width of the simulation environment
     * @param height height of the simulation environment
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public synchronized void setEnvironmentDimension(int width, int height) throws CommandExecException {
        CommandUtility.sendEnvironmentDimension(width, height);
    }

    /**
     * Command to delete the visual reference (arrow) which indicates a location
     * (waypoint) within the server environment.
     *
     * @param x x abscissa of the waypoint
     * @param y y ordinate of the waypoint
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public synchronized void deleteWaypoint(double x, double y) throws CommandExecException {
        CommandUtility.sendDelWaypoint(x, y);
    }

    /**
     * Check if the simulation on the server has started.
     *
     * @return "true": has started; "false": not started yet.
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public boolean requestGameStatus() throws CommandExecException {
        return CommandUtility.requestGameStatus();
    }

    public void setDeliverySpot(double x, double y) {
        deliverySpot = new WorldPoint(x, y);
    }

    public WorldPoint getDeliverySpot() {
        return deliverySpot;
    }

    /**
     * Returns the number of creatures and other Things currently at the
     * simulation environment.
     *
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void getNumberOfEntities() throws CommandExecException {
        CommandUtility.sendGetNumberOfEntities();
    }

    // EFJC - 2013-06
    /**
     * Returns the number of creatures at the
     * simulation environment.
     *
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized int getNumberOfCreatures() throws CommandExecException {
        int n = 0;
        StringTokenizer st = CommandUtility.sendGetNumberOfEntities();
        if (st.hasMoreTokens()) {
            n = Integer.parseInt(st.nextToken());
        }
        return n;
    }

    /**
     * Returns the creatures and other Things currently at the simulation
     * environment.
     *
     * @return list of all entities (creatures, food, jewel and bricks) that
     * exist in the world
     *
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized List<Thing> getWorldEntities() throws CommandExecException {

        List<Thing> all = new ArrayList<Thing>();

        StringTokenizer st;
        String ret = "";
        String command = "";

        double x1 = 0.0, x2 = 0.0, y1 = 0.0, y2 = 0.0, thingPitch = 0.0, hardness = 0.0, energy = 0.0, taste = 0.0;
        String thingName = "none";
        int category = 0;
        int isOccluded = 0;
        int number = 0; //number of Thing
        String thingColor = "No color yet";
        
        logger.info("----------- getWorldEntities -----------");

        st = CommandUtility.sendGetWorldEntities();

        if (!st.hasMoreTokens()) {
            logger.log(Level.SEVERE, "Error - missing number of things!");
        } else {
            command = st.nextToken();
            number = Integer.parseInt(command);
        }

        for (int i = 0; i <= number - 1; i++) { //loop to create Things
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing name!");
            } else {
                command = st.nextToken();
                if (command.equals("||")) {
                    command = st.nextToken();
                }
                thingName = command;

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing category!");
            } else {
                command = st.nextToken();
                category = Integer.parseInt(command);

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing if occluded!");
            } else {
                command = st.nextToken();
                isOccluded = Integer.parseInt(command);

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing X1!");
            } else {
                command = st.nextToken();
                x1 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing X2!");
            } else {
                command = st.nextToken();
                x2 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing Y1!");
            } else {
                command = st.nextToken();
                y1 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing Y2!");
            } else {
                command = st.nextToken();
                y2 = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing pitch !");
            } else {
                command = st.nextToken();
                thingPitch = Double.parseDouble(command);
                thingPitch = Math.toRadians(thingPitch) + Constants.M_PI_2;

            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing hardness!");
            } else {
                command = st.nextToken();
                hardness = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing energy!");
            } else {
                command = st.nextToken();
                energy = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing taste!");
            } else {
                command = st.nextToken();
                taste = Double.parseDouble(command);
            }
            if (!st.hasMoreTokens()) {
                logger.log(Level.SEVERE, "Error - missing color of thing!");
            } else {
                command = st.nextToken();
                thingColor = command;
            }

            all.add(createThing(thingName, category, isOccluded, thingColor, x1, y1, x2, y2, thingPitch, hardness, energy, taste));
        }
        return all;
    }

    private static Thing createThing(String name, int category, int ifIsOccluded, String thingColor, double x1, double y1, double x2, double y2, double thingPitch, double hardness, double energy, double taste) {

        return new Thing(name, category, x1, y1, x2, y2, thingColor, ifIsOccluded, thingPitch, hardness, energy, taste) {
        };
    }

    /**
     * Creates a new brick entity.
     *
     * @param type 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @param x abscissa of the location of the brick
     * @param y ordinate of the location of the brick
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void createBrick(int type, double X1, double Y1, double X2, double Y2) throws CommandExecException {
        CommandUtility.sendNewBrick(type, X1, Y1, X2, Y2);
    }

    /**
     * Creates a new cage.
     *
     * @param x abscissa of the location of the cage
     * @param y ordinate of the location of the cage
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void createCage(double X, double Y) throws CommandExecException {
        CommandUtility.sendNewCage(X, Y);
    }

    /**
     * Delete all entities in the world (Creatures and objects).
     *
     * @throws CommandExecException
     */
    public synchronized void reset() throws CommandExecException {
        CommandUtility.sendResetWorld();
    }

    /**
     * Creates a new food entity.
     *
     * @param type 0-perishable 1-non-perishable
     * @param x abscissa of the location of the food
     * @param y ordinate of the location of the food
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void createFood(int type, double X, double Y) throws CommandExecException {
        CommandUtility.sendNewFood(type, X, Y);
    }
    
    
    public static synchronized void grow(){
        grow(Constants.TIMEFRAME);        
    }
    
    public static synchronized void grow(int time) {
        try {
            if(time <= 0){
                time = Constants.TIMEFRAME;
            }
            getDimensionAndDeliverySpot();
            ResourcesGenerator rg = new ResourcesGenerator(time, World.environmentWidth, World.environmentHeight, World.deliverySpot.getX(), World.deliverySpot.getY());
            rg.start();
        } catch (CommandExecException ex) {
            Logger.getLogger(World.class.getName()).log(Level.SEVERE, null, ex);
        }
    }


    /**
     * Creates several food at once.
     *
     * @param type 0-perishable 1-non-perishable
     * @param number number of food entities to create
     * @param X_Y text with the list of location where a new food must be
     * created
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void createFoodInBatch(int type, int number, String X_Y) throws CommandExecException {
        CommandUtility.sendNewFoodInBatch(type, number, X_Y);
    }

    /**
     * Creates a new jewel entity.
     *
     * @param type 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @param x abscissa of the location of the jewel
     * @param y ordinate of the location of the jewel
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void createJewel(int type, double X, double Y) throws CommandExecException {
        CommandUtility.sendNewJewel(type, X, Y);
    }

    /**
     * Creates several jewels at once.
     *
     * @param type 0-Red 1-Green 2-Blue 3-Yellow 4-Magenta 5-White
     * @param number number of jewel entities to create
     * @param X_Y text with the list of location where a new jewel must be
     * created
     * @throws CommandExecException An exception is thrown in case of missing or
     * invalid parameter
     */
    public static synchronized void createJewelsInBatch(int type, int number, String X_Y) throws CommandExecException {
        CommandUtility.sendNewJewelsInBatch(type, number, X_Y);
    }
}
